package com.inuker.bluetooth;

import android.Manifest;
import android.app.Activity;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.util.Log;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.clj.fastble.utils.HexUtil;
import com.inuker.bluetooth.dialog.DeviceLinkSleepDialogHelper;
import com.inuker.bluetooth.dialog.WifiSetDialogHelper;
import com.inuker.bluetooth.library.Constants;
import com.inuker.bluetooth.library.connect.listener.BluetoothStateListener;
import com.inuker.bluetooth.library.connect.options.BleConnectOptions;
import com.inuker.bluetooth.library.connect.response.BleConnectResponse;
import com.inuker.bluetooth.library.connect.response.BleMtuResponse;
import com.inuker.bluetooth.library.connect.response.BleNotifyResponse;
import com.inuker.bluetooth.library.connect.response.BleWriteResponse;
import com.inuker.bluetooth.library.model.BleGattProfile;
import com.inuker.bluetooth.library.search.SearchRequest;
import com.inuker.bluetooth.library.search.SearchResult;
import com.inuker.bluetooth.library.search.response.SearchResponse;
import com.inuker.bluetooth.library.utils.BluetoothLog;
import com.inuker.bluetooth.library.utils.ByteUtils;
import com.inuker.bluetooth.sleep.BluetoothUtil;
import com.inuker.bluetooth.sleep.SleepDataParse;
import com.inuker.bluetooth.view.PullRefreshListView;
import com.inuker.bluetooth.view.PullToRefreshFrameLayout;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import pub.devrel.easypermissions.EasyPermissions;

import static com.inuker.bluetooth.library.Constants.REQUEST_SUCCESS;

public class MainActivity extends Activity implements  DeviceListAdapter.OnItemClickListener, EasyPermissions.PermissionCallbacks {

//    private static final String MAC = "B0:D5:9D:6F:E7:A5";

    private PullToRefreshFrameLayout mRefreshLayout;
    private PullRefreshListView mListView;
    private DeviceListAdapter mAdapter;
    private TextView mTvTitle;
    private TextView tvVersion;
    private ProgressBar pbar;

    private List<SearchResult> mDevices;
    private UUID mService = UUID.fromString("0000fff0-0000-1000-8000-00805f9b34fb");

    //手机发往设备
    private UUID mCharacter1 =UUID.fromString("0000fff1-0000-1000-8000-00805f9b34fb");
    //设备发往手机
    private  UUID mCharacter2 =UUID.fromString("0000fff2-0000-1000-8000-00805f9b34fb");

    private UUID ld_mService = UUID.fromString("000000f4-0000-1000-8000-00805f9b34fb");
    //手机发往设备
    private UUID ld_mCharacter1 =UUID.fromString("0000f401-0000-1000-8000-00805f9b34fb");
    //设备发往手机
    private  UUID ld_mCharacter2 =UUID.fromString("0000f301-0000-1000-8000-00805f9b34fb");

    private MainActivity inst;

    //所需权限
    public final static String[] permissions = new String[]{
            Manifest.permission.BLUETOOTH,
            Manifest.permission.ACCESS_FINE_LOCATION,
            Manifest.permission.ACCESS_COARSE_LOCATION
    };

    private final int PERMISSION_REQUEST_CODE = 101;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        mDevices = new ArrayList<>();
        inst =this;
        mTvTitle = findViewById(R.id.title);
        pbar = findViewById(R.id.mainpbar);
        mRefreshLayout = findViewById(R.id.pulllayout);
        tvVersion = findViewById(R.id.version);
        tvVersion.setText("v" + BuildConfig.VERSION_NAME);

        mListView = mRefreshLayout.getPullToRefreshListView();
        mAdapter = new DeviceListAdapter(this);
        mAdapter.setOnItemClickListener(this);
        mListView.setAdapter(mAdapter);

        mListView.setOnRefreshListener(new PullRefreshListView.OnRefreshListener() {

            @Override
            public void onRefresh() {
                searchDevice();
            }

        });

        if (EasyPermissions.hasPermissions(this, permissions)) {
            permissionGranted();
        } else {
            mRefreshLayout.showState(AppConstants.ALLOW_PULL_IN_EMPTY_PAGE);

            EasyPermissions.requestPermissions(this, getString(R.string.permission_request_notice),
                    PERMISSION_REQUEST_CODE, permissions);
        }

    }

    private void permissionGranted() {
        Log.i(TAG, "permission granted...");
        ClientManager.getClient().registerBluetoothStateListener(mBluetoothStateListener);

        searchDevice();

        ClientManager.getClient().registerBluetoothStateListener(new BluetoothStateListener() {
            @Override
            public void onBluetoothStateChanged(boolean openOrClosed) {
                BluetoothLog.v(String.format("onBluetoothStateChanged %b", openOrClosed));
            }
        });
    }

    private void searchDevice() {
        if (ClientManager.getClient().isBluetoothOpened()) {
            SearchRequest request = new SearchRequest.Builder()
                    .searchBluetoothLeDevice(5000, 2)// 先扫BLE设备3次，每次5s
//                    .searchBluetoothClassicDevice(5000) // 再扫经典蓝牙5s
//                    .searchBluetoothLeDevice(2000)// 再扫BLE设备2s
                    .build();

            ClientManager.getClient().search(request, mSearchResponse);
        }else {
            CommonUtils.toast(getString(R.string.ble_device_5));
            ClientManager.getClient().openBluetooth();
        }
    }

    private final BluetoothStateListener mBluetoothStateListener = new BluetoothStateListener() {
        @Override
        public void onBluetoothStateChanged(boolean openOrClosed) {
            if (openOrClosed){
                searchDevice();
            }
        }

    };

    private final SearchResponse mSearchResponse = new SearchResponse() {
        @Override
        public void onSearchStarted() {
            BluetoothLog.w("MainActivity.onSearchStarted");
            mListView.onRefreshComplete(true);
            mRefreshLayout.showState(AppConstants.LIST);
            pbar.setVisibility(View.VISIBLE);
            mTvTitle.setText(R.string.string_refreshing);
            mDevices.clear();
        }

        @Override
        public void onDeviceFounded(SearchResult device) {
//            BluetoothLog.w("MainActivity.onDeviceFounded " + device.device.getAddress());

            if (!mDevices.contains(device) &&(device.getName().contains("LZ-OTA") || device.getName().contains("ED719W"))) {
                mDevices.add(device);
                mAdapter.setDataList(mDevices);

//                Beacon beacon = new Beacon(device.scanRecord);
//                BluetoothLog.v(String.format("beacon for %s\n%s", device.getAddress(), beacon.toString()));

//                BeaconItem beaconItem = null;
//                BeaconParser beaconParser = new BeaconParser(beaconItem);
//                int firstByte = beaconParser.readByte(); // 读取第1个字节
//                int secondByte = beaconParser.readByte(); // 读取第2个字节
//                int productId = beaconParser.readShort(); // 读取第3,4个字节
//                boolean bit1 = beaconParser.getBit(firstByte, 0); // 获取第1字节的第1bit
//                boolean bit2 = beaconParser.getBit(firstByte, 1); // 获取第1字节的第2bit
//                beaconParser.setPosition(0); // 将读取起点设置到第1字节处
            }

            if (mDevices.size() > 0) {
                mRefreshLayout.showState(AppConstants.LIST);
            }
        }

        @Override
        public void onSearchStopped() {
            BluetoothLog.w("MainActivity.onSearchStopped");
            mListView.onRefreshComplete(true);
            mRefreshLayout.showState(AppConstants.LIST);
            pbar.setVisibility(View.GONE);
            mTvTitle.setText(R.string.devices);
        }

        @Override
        public void onSearchCanceled() {
            BluetoothLog.w("MainActivity.onSearchCanceled");
            pbar.setVisibility(View.GONE);
            mListView.onRefreshComplete(true);
            mRefreshLayout.showState(AppConstants.LIST);

            mTvTitle.setText(R.string.devices);
        }
    };

    @Override
    protected void onPause() {
        super.onPause();
        ClientManager.getClient().stopSearch();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        ClientManager.getClient().stopSearch();
        ClientManager.getClient().unregisterBluetoothStateListener(mBluetoothStateListener);
    }

    String bt_mac ="";
    @Override
    public void onItemClick(String mac, final String name) {
        if (!mac.equals("")&& bt_mac.equals("")){
             bt_mac=mac;
            ClientManager.getClient().requestMtu(bt_mac, 500, mMtuResponse);
            //搜索到了绑定设备 开始配对
            BleConnectOptions options = new BleConnectOptions.Builder()
                    .setConnectRetry(3)   // 连接如果失败重试3次
                    .setConnectTimeout(30000)   // 连接超时30s
                    .setServiceDiscoverRetry(3)  // 发现服务如果失败重试3次
                    .setServiceDiscoverTimeout(20000)  // 发现服务超时20s
                    .build();

            ClientManager.getClient().connect(bt_mac, options, new BleConnectResponse() {
                @Override
                public void onResponse(int code, BleGattProfile data) {
                    if (code == Constants.REQUEST_SUCCESS) {
                        //连接成功
                        DeviceLinkSleepDialogHelper.showDialog(inst,name, new DeviceLinkSleepDialogHelper.ClickListener() {
                            @Override
                            public void onClick(String name, String Password, String type) {
                                if (name != null && Password != null) {
                                    pbar.setVisibility(View.VISIBLE);
                                    if ("LZ-OTA".contains(name)){
                                        sendMsg(name, Password);
                                    }else {
                                        //休眠2秒
                                        sendMsg2(name, Password);
                                    }

                                }
                            }
                        });
                    }else {
                        bt_mac="";
                        //连接失败
                        WifiSetDialogHelper.showDialog(inst, 2);

                    }
                }
            });
        }

    }

    private final BleMtuResponse mMtuResponse = new BleMtuResponse() {
        @Override
        public void onResponse(int code, Integer data) {
            if (code == REQUEST_SUCCESS) {
                CommonUtils.toast("request mtu success,mtu = " + data);
            } else {
                CommonUtils.toast("request mtu failed");
            }
        }
    };
    //发送数据
    private void sendMsg(String name, String Password) {
        ClientManager.getClient().notify(bt_mac, mService, mCharacter2, mNotifyRsp);
        //组装数据
        byte[] signData = SleepDataParse.parseValue4(name, Password);
        Log.d(TAG, "发送配网数据: " + HexUtil.formatHexString(signData, false));
        int maxPacketSize = 20; // 最大包大小
        if (signData.length <= maxPacketSize) {
            ClientManager.getClient().write(bt_mac, mService, mCharacter1, signData, mWriteRsp);
            Log.d(TAG, "总数据: " + HexUtil.formatHexString(signData, false));
        } else {
            // 数据需要分包发送
            int numPackets = (int) Math.ceil(signData.length / (double) maxPacketSize);
            for (int i = 0; i < numPackets; i++) {
                int start = i * maxPacketSize;
                int end = Math.min(signData.length, start + maxPacketSize);
                byte[] packetData = Arrays.copyOfRange(signData, start, end);
                ClientManager.getClient().write(bt_mac, mService, mCharacter1, packetData, mWriteRsp);
                Log.d(TAG, "分包数据: " + HexUtil.formatHexString(packetData, false));

                try {
                    Thread.sleep(100); // 100 毫秒延时
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
    }


    private BleWriteResponse mWriteRsp = new BleWriteResponse() {
        @Override
        public void onResponse(int code) {
            if (code == Constants.REQUEST_SUCCESS) {
                BluetoothLog.v(TAG + "发送成功");
            } else {
                BluetoothLog.v(TAG + "发送失败");
            }
        }
    };

    private String TAG = "DeviceLinkageActivity";

    private BleNotifyResponse mNotifyRsp = new BleNotifyResponse() {
        @Override
        public void onNotify(UUID service, UUID character, byte[] value) {
            if (service.equals(ld_mService) && character.equals(ld_mCharacter2)) {
                Log.d(TAG, String.format("%s", ByteUtils.byteToString(value)));
                BluetoothLog.v(TAG + "收到通知");
                String substring = String.format("%s", ByteUtils.byteToString(value)).substring(8, 10);
                BluetoothLog.v(TAG + substring);
                pbar.setVisibility(View.GONE);
                if (substring.equals("00")) {
                    WifiSetDialogHelper.showDialog(inst, 1);
                } else if (substring.equals("01")) {
                    WifiSetDialogHelper.showDialog(inst, 2);
                } else {
                    BluetoothLog.v(TAG + "收到其他通知");
                }
                bt_mac="";
            }
        }

        @Override
        public void onResponse(int code) {
            if (code == Constants.REQUEST_SUCCESS) {
                BluetoothLog.v(TAG + "开启通知");
            } else {
                BluetoothLog.v(TAG + "开启通知失败");
            }
        }
    };
    //发送数据
    private void sendMsg2(String name, String Password) {
        ClientManager.getClient().notify(bt_mac, ld_mService, ld_mCharacter2, mNotifyRsp2);
        Log.d(TAG, "wifi名称: " + name+"wifi密码: " + Password);
        //名称以及密码以双引号包含，中间加逗号
        String combinedString = name + "|" + Password;
        String str = BluetoothUtil.strTo16(combinedString);
        byte[] data =  HexUtil.hexStringToBytes(str);
        // 计算数据包的总数和当前包的序号
        int totalPackets = (int)Math.ceil((double)data.length / 18);
            // 数据需要分包发送
            for (int i = 0; i < totalPackets; i++) {
                // 根据规则设置 Len 字段
                int lenValue = (totalPackets << 4) | i+1;
                byte[] packetData = SleepDataParse.parseValue5(lenValue, data, i);
                ClientManager.getClient().write(bt_mac, ld_mService, ld_mCharacter1, packetData, mWriteRsp);
                Log.d(TAG, "分包数据: " + HexUtil.formatHexString(packetData, false));
                try {
                    Thread.sleep(100); // 100 毫秒延时
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
    }
    private BleNotifyResponse mNotifyRsp2 = new BleNotifyResponse() {
        @Override
        public void onNotify(UUID service, UUID character, byte[] value) {
//        BluetoothLog.v(TAG + "收到通知service  "+service.toString()+"  character  "+character.toString()+"  value  "+String.format("%s", ByteUtils.byteToString(value)));
            if (service.equals(ld_mService) && character.equals(ld_mCharacter2)) {
                Log.d(TAG, String.format("%s", ByteUtils.byteToString(value)));
                String substring = String.format("%s", ByteUtils.byteToString(value));
                if (substring.equals("03")) {
                    WifiSetDialogHelper.showDialog(inst, 1);
                    pbar.setVisibility(View.GONE);
                    bt_mac="";
                } else if (substring.equals("02")|| substring.equals("01")) {
                    WifiSetDialogHelper.showDialog(inst, 2);
                    pbar.setVisibility(View.GONE);
                    bt_mac="";
                } else {
                    bt_mac="";
                    BluetoothLog.v(TAG + "收到其他通知");
                }

            }
        }

        @Override
        public void onResponse(int code) {
            if (code == Constants.REQUEST_SUCCESS) {
                BluetoothLog.v(TAG + "开启通知");
            } else {
                BluetoothLog.v(TAG + "开启通知失败");
            }
        }
    };

    @Override
    public void onPermissionsGranted(int requestCode, @NonNull List<String> perms) {
        permissionGranted();
    }

    @Override
    public void onPermissionsDenied(int requestCode, @NonNull List<String> perms) {
        //重新申请权限
        EasyPermissions.requestPermissions(this, getString(R.string.permission_request_notice),
                PERMISSION_REQUEST_CODE, permissions);
    }
}
