package com.inuker.bluetooth.sleep;

public class BluetoothUtil {


    /**
     * Hex转byte[]，两种情况，Hex长度为奇数最后一个字符会被舍去
     */
    public static byte[] hexTobytes(String hex) {
        if (hex.length() < 1) {
            return null;
        } else {
            byte[] result = new byte[hex.length() / 2];
            int j = 0;
            for(int i = 0; i < hex.length(); i+=2) {
                result[j++] = (byte) Integer.parseInt(hex.substring(i,i+2), 16);
            }
            return result;
        }
    }

    // Helper method to extract bytes from byte array.
    public static byte[] extractBytes(byte[] scanRecord, int start, int length) {
        byte[] bytes = new byte[length];
        System.arraycopy(scanRecord, start, bytes, 0, length);
        return bytes;
    }

    // 格式化UUID
    public static String parseUUID(String data) {
        String uuid = "";
        if (data.length() == 32) {
            uuid = data.substring(0, 8) + "-"
                    + data.substring(8, 12) + "-"
                    + data.substring(12, 16) + "-"
                    + data.substring(16, 20) + "-"
                    + data.substring(20);
        } else {
//            showTips(getString(R.string.toast_uuid_not_found));
        }
        return uuid;
    }

    /**
     * byte数组转化为string
     */
    static final char[] hexArray = "0123456789ABCDEF".toCharArray();
    public static String bytesToHex(byte[] bytes) {
        char[] hexChars = new char[bytes.length * 2];
        for (int j = 0; j < bytes.length; j++) {
            int v = bytes[j] & 0xFF;
            hexChars[j * 2] = hexArray[v >>> 4];
            hexChars[j * 2 + 1] = hexArray[v & 0x0F];
        }
        return new String(hexChars);
    }

    // 根据RSSI计算距离
    public static double rssi2distance(int rssi) {
        int iRssi = Math.abs(rssi);
        // 发射端和接收端相隔1米时的信号强度
        int A = 59;
        // 环境噪声衰减因子
        double n = 2.0;
        double power = (iRssi - A) / (10 * n);
        return Math.pow(10, power);
    }

    public static byte[] hexStringToByteArray(String hexString) {
        hexString = hexString.replaceAll(" ", "");
        int len = hexString.length();
        byte[] bytes = new byte[len / 2];
        for (int i = 0; i < len; i += 2) {
            // 两位一组，表示一个字节,把这样表示的16进制字符串，还原成一个字节
            bytes[i / 2] = (byte) ((Character.digit(hexString.charAt(i), 16) << 4) + Character
                    .digit(hexString.charAt(i + 1), 16));
        }
        return bytes;
    }

    /**
     * 16进制转换成为string类型字符串
     * @param s
     * @return
     */
    public static String hexStringToString(String s) {
        if (s == null || s.equals("")) {
            return null;
        }
        s = s.replace(" ", "");
        byte[] baKeyword = new byte[s.length() / 2];
        for (int i = 0; i < baKeyword.length; i++) {
            try {
                baKeyword[i] = (byte) (0xff & Integer.parseInt(s.substring(i * 2, i * 2 + 2), 16));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        try {
            s = new String(baKeyword, "UTF-8");
            new String();
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        return s;
    }

    /**
     * 字符串转化成为16进制字符串
     * @param s
     * @return
     */
    public static String strTo16(String s) {
        String str = "";
        for (int i = 0; i < s.length(); i++) {
            int ch = (int) s.charAt(i);
            String s4 = Integer.toHexString(ch);
            str = str + s4;
        }
        return str.toUpperCase();
    }

    public static String intToTwoBytesHexString(int number) {
        // 将整数转换为两个字节的十六进制表示
        byte[] bytes = new byte[2];
        bytes[0] = (byte) ((number >> 8) & 0xFF);  // 高位字节
        bytes[1] = (byte) (number & 0xFF);         // 低位字节

        // 将字节数组转换为十六进制字符串
        StringBuilder hexString = new StringBuilder();
        for (byte b : bytes) {
            hexString.append(String.format("%02X", b));
        }
        return hexString.toString();
    }

    /**
     * 10进制转化成为16进制字符串
     * @return
     */
    public static String hex10To16(String valueTen) {
        return String.format("%04X",valueTen);//自定义保留几位
    }
    /**
     * 16进制字符串转化成为10进制
     * @return
     */
    public static int hexToDecimal(String hexString) {
        try {
            return Integer.parseInt(hexString, 16);
        } catch (NumberFormatException e) {
            e.printStackTrace();
            return 0;
        }
    }


}
